#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
###
# Application: wah!cade
# File:        gst_video.py
# Description: gstreamer video widget
# Copyright (c) 2007   Andy Balcombe <http://www.anti-particle.com>
#       Taken from gstreamer docs example code
###
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

import pygtk
pygtk.require('2.0')

import gobject

import pygst
pygst.require('0.10')
import gst
import gst.interfaces
import gtk

class GstPlayer:
    def __init__(self, videowidget):
        """initialise player class"""
        self.playing = False
        self.player = gst.element_factory_make("playbin", "player")
        self.videowidget = videowidget
        self.on_eos = False
        #
        bus = self.player.get_bus()
        bus.enable_sync_message_emission()
        bus.add_signal_watch()
        bus.connect('sync-message::element', self.on_sync_message)
        bus.connect('message', self.on_message)

    def on_sync_message(self, bus, message):
        """message from player?"""
        if message.structure is None:
            return
        if message.structure.get_name() == 'prepare-xwindow-id':
            self.videowidget.set_sink(message.src)
            #message.src.set_property('force-aspect-ratio', True)
            #self.player.set_property('volume', 0)
            #print "vol=", self.player.get_property('volume')
            
    def on_message(self, bus, message):
        """message from player?"""
        t = message.type
        if t == gst.MESSAGE_ERROR:
            err, debug = message.parse_error()
            print "Error: %s" % err, debug
            if self.on_eos:
                self.on_eos()
            self.playing = False
        elif t == gst.MESSAGE_EOS:
            if self.on_eos:
                self.on_eos()
            self.playing = False

    def set_location(self, location):
        """set filename"""
        self.player.set_property('uri', location)

    def query_position(self):
        """Returns a (position, duration) tuple"""
        #get position
        try:
            position, format = self.player.query_position(gst.FORMAT_TIME)
        except:
            position = gst.CLOCK_TIME_NONE
        #get duration
        try:
            duration, format = self.player.query_duration(gst.FORMAT_TIME)
        except:
            duration = gst.CLOCK_TIME_NONE
        #done
        return (position, duration)

    def seek(self, location):
        """
        @param location: time to seek to, in nanoseconds
        """
        gst.debug("seeking to %r" % location)
        event = gst.event_new_seek(1.0, gst.FORMAT_TIME,
            gst.SEEK_FLAG_FLUSH | gst.SEEK_FLAG_ACCURATE,
            gst.SEEK_TYPE_SET, location,
            gst.SEEK_TYPE_NONE, 0)
        res = self.player.send_event(event)
        if res:
            gst.info("setting new stream time to 0")
            self.player.set_new_stream_time(0L)
        else:
            gst.error("seek to %r failed" % location)

    def pause(self):
        """pause player"""
        gst.info("pausing player")
        self.player.set_state(gst.STATE_PAUSED)
        self.playing = False

    def play(self):
        """play file"""
        gst.info("playing player")
        self.player.set_state(gst.STATE_PLAYING)
        self.playing = True
        
    def stop(self):
        """stop playing"""
        self.player.set_state(gst.STATE_NULL)
        gst.info("stopped player")

    def get_state(self, timeout=1):
        """get state (playing / paused / etc)"""
        return self.player.get_state(timeout=timeout)

    def is_playing(self):
        """is file playing"""
        return self.playing
    
    def set_volume(self, volume_level):
        """set the volume level"""
        self.player.set_property('volume', volume_level)
        gst.info("changed volume to [%s]" % volume_level)

    
class VideoWidget(gtk.DrawingArea):
    def __init__(self):
        """initialise video widget"""
        gtk.DrawingArea.__init__(self)
        self.imagesink = None
        self.unset_flags(gtk.DOUBLE_BUFFERED)

    def do_expose_event(self, event):
        """expose"""
        #print "gst event=",event
        if self.imagesink:
            self.imagesink.expose()
            return False
        else:
            return True

    def set_sink(self, sink):
        """set sink"""
        assert self.window.xid
        self.imagesink = sink
        self.imagesink.set_xwindow_id(self.window.xid)

        
class GstVideo:
    def __init__(self, videowidget):
        """initialise video widget"""
        #self.vid_filename = ''
        self.videowidget = videowidget
        self.player = GstPlayer(self.videowidget)
        self.player.on_eos = lambda *x: self.on_eos()
        self.update_id = -1
        self.changed_id = -1
        self.seek_timeout_id = -1
        self.p_position = gst.CLOCK_TIME_NONE
        self.p_duration = gst.CLOCK_TIME_NONE

    def on_eos(self):
        """video file has finished"""
        #play it again
        self.player.seek(0L)
        self.player.play()
        
    def play(self, vid_filename):
        """play given video file"""
        #self.vid_filename = vid_filename
        self.player.set_location('file://%s' % vid_filename)
        self.player.play()

    def stop(self):
        """stop playing"""
        self.player.stop()
        
    def close(self):
        """close"""
        pass
    
    def set_volume(self, volume_level):
        """change volume level"""
        self.player.set_volume((volume_level / 100.0))
        
